package be.iminds.ilabt.jfed.experiment.setup.config;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nullable;
import java.util.Objects;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class WaitForReady {
    protected final long maxTimeMin;
    protected final long extraWaitTimeSeconds;
    protected final boolean enabled;

    public WaitForReady() {
        this.maxTimeMin = 20;
        this.extraWaitTimeSeconds = 0;
        this.enabled = true;
    }
    @JsonCreator
    public WaitForReady(@Nullable @JsonProperty("maxTimeMin") Long maxTimeMin,
                        @Nullable @JsonProperty("extraWaitTimeSeconds") Long extraWaitTimeSeconds,
                        @Nullable @JsonProperty("enabled") Boolean enabled) {
        this.maxTimeMin = maxTimeMin == null ? new WaitForReady().getMaxTimeMin() : maxTimeMin;
        this.extraWaitTimeSeconds = extraWaitTimeSeconds == null ? new WaitForReady().getExtraWaitTimeSeconds() : extraWaitTimeSeconds;
        this.enabled = enabled == null ? new WaitForReady().isEnabled() : enabled;
    }

    @JsonIgnore
    public boolean isValid() {
        if (!enabled)
            return true;

        if (maxTimeMin < 0)
            throw new IllegalArgumentException("Config Error: waitForReady.maxTimeMin < 0 makes no sense");

        return true;
    }


    @JsonProperty
    public long getMaxTimeMin() {
        return maxTimeMin;
    }


    @JsonProperty
    public long getExtraWaitTimeSeconds() {
        return extraWaitTimeSeconds;
    }

    @JsonProperty
    public boolean isEnabled() {
        return enabled;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        WaitForReady that = (WaitForReady) o;
        return maxTimeMin == that.maxTimeMin &&
                extraWaitTimeSeconds == that.extraWaitTimeSeconds &&
                enabled == that.enabled;
    }

    @Override
    public int hashCode() {
        return Objects.hash(maxTimeMin, extraWaitTimeSeconds, enabled);
    }
}
